==> mobroute-0.5.0.tar.gz: mobroute-v0.5.0/csa/csaloop.go <==
package csa

import (
	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"git.sr.ht/~mil/mobroute/util/utillog"
)

// ExecuteCSAMainLoop is the main logic loop for the CSA algorithm
// returns a map containing SIDU-to-connectionptr AND another map
// for SIDU-to-arrivaltime
func ExecuteCSAMainLoop(
	conns *[]dbt.Connection,
	transfers *[]dbt.Transfer,
	stopWalkcostsOrigin map[string]uint,
	originTimeSeconds int,
	maxTransferSeconds, maxNTransfers uint,
) map[string]ArrivalDetails {
	utillog.Info("CSA Main Loop Begin")
	defer utillog.Info("CSA Main Loop Complete")

	var (
		maxNTrips               = maxNTransfers + 1
		transfersMap            = buildTransfersMap(transfers)
		mapStopToArrivalDetails = InitializeMapStopToArrivalDetails(
			utilfuncs.MapValueAdd(stopWalkcostsOrigin, originTimeSeconds),
		)
	)
	for _, c := range *conns {
		var (
			fromConnection = mapStopToArrivalDetails[c.FromSSIDU].Connection

			connPossibleDirect            = fromConnection == nil || fromConnection.TripID == c.TripID
			connPossibleTransferConnected = TransferPossibleBetweenConnections(
				transfersMap, mapStopToArrivalDetails[c.FromSSIDU].Connection,
				&c, maxTransferSeconds,
			)
			connPossible = connPossibleDirect || connPossibleTransferConnected

			connOriginDepartsLater        = c.FromST >= mapStopToArrivalDetails[c.FromSSIDU].Time
			connDestinationArrivesEarlier = c.ToST < mapStopToArrivalDetails[c.ToSSIDU].Time
		)
		if connOriginDepartsLater && connDestinationArrivesEarlier && connPossible {
			cc := c

			newNTransfers := func() int {
				if connPossibleDirect {
					return int(utilfuncs.MaxInt(1, mapStopToArrivalDetails[c.FromSSIDU].NTrips))
				} else if mapStopToArrivalDetails[c.FromSSIDU].NTrips < maxNTrips {
					return int(mapStopToArrivalDetails[c.FromSSIDU].NTrips + 1)
				} else {
					return -1
				}
			}()

			if newNTransfers == -1 {
				continue
			}

			mapStopToArrivalDetails[c.ToSSIDU] = ArrivalDetails{
				Time:       c.ToST,
				Connection: &cc,
				NTrips:     uint(newNTransfers),
			}
			for transferToSSIDU, transferToStopCostSeconds := range transfersMap[c.ToSSIDU] {
				var (
					transferFromTime = c.FromST
					transferToST     = c.ToST + int(transferToStopCostSeconds)
					departsLater     = transferFromTime >= mapStopToArrivalDetails[c.FromSSIDU].Time
					arrivesEarlier   = transferToST < mapStopToArrivalDetails[transferToSSIDU].Time
					tPoss            = transferToStopCostSeconds < maxTransferSeconds
				)
				if departsLater && arrivesEarlier && tPoss && mapStopToArrivalDetails[c.FromSSIDU].NTrips < maxNTrips {
					mapStopToArrivalDetails[transferToSSIDU] = ArrivalDetails{
						Time:       transferToST,
						Connection: &cc,
						NTrips:     mapStopToArrivalDetails[c.FromSSIDU].NTrips + 1,
					}
				}
			}
		}
	}
	utillog.Warnf("Completed main CSA loop %v", mapStopToArrivalDetails)
	return mapStopToArrivalDetails
}

==> mobroute-0.7.0.tar.gz: mobroute-v0.7.0/csa/csaloop.go <==
package csa

import (
	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
	"git.sr.ht/~mil/mobroute/util/utillog"
)

// ExecuteCSAMainLoop is the main logic loop for the CSA algorithm
// returns a map containing SIDU-to-connectionptr AND another map
// for SIDU-to-arrivaltime
func ExecuteCSAMainLoop(
	conns *[]dbt.Connection,
	transfers *[]dbt.Transfer,
	stopWalkcostsOrigin map[string]uint,
	originTimeSeconds int,
	maxTransferSeconds, maxNTransfers uint,
) map[string]ArrivalDetails {
	utillog.Info("CSA Main Loop Begin")
	defer utillog.Info("CSA Main Loop Complete")

	var (
		maxNTrips               = maxNTransfers + 1
		transfersMap            = buildTransfersMap(transfers)
		mapStopToArrivalDetails = InitializeMapStopToArrivalDetails(
			utilfuncs.MapValueAdd(stopWalkcostsOrigin, originTimeSeconds),
		)
	)
	for _, c := range *conns {
		var (
			fromConnection = mapStopToArrivalDetails[c.FromSSIDU].Connection

			connPossibleDirect            = fromConnection == nil || fromConnection.TripID == c.TripID
			connPossibleTransferConnected = TransferPossibleBetweenConnections(
				transfersMap, mapStopToArrivalDetails[c.FromSSIDU].Connection,
				&c, maxTransferSeconds,
			)
			connPossible = connPossibleDirect || connPossibleTransferConnected

			connOriginDepartsLater        = c.FromST >= mapStopToArrivalDetails[c.FromSSIDU].Time
			connDestinationArrivesEarlier = c.ToST < mapStopToArrivalDetails[c.ToSSIDU].Time
		)
		if connOriginDepartsLater && connDestinationArrivesEarlier && connPossible {
			cc := c

			var newNTrips int
			if connPossibleDirect {
				newNTrips = int(utilfuncs.MaxInt(1, mapStopToArrivalDetails[c.FromSSIDU].NTrips))
			} else if mapStopToArrivalDetails[c.FromSSIDU].NTrips < maxNTrips {
				newNTrips = int(mapStopToArrivalDetails[c.FromSSIDU].NTrips + 1)
			} else {
				continue
			}

			mapStopToArrivalDetails[c.ToSSIDU] = ArrivalDetails{
				Time:       c.ToST,
				Connection: &cc,
				NTrips:     uint(newNTrips),
			}
			for transferToSSIDU, transferToStopCostSeconds := range transfersMap[c.ToSSIDU] {
				var (
					transferFromTime = c.FromST
					transferToST     = c.ToST + int(transferToStopCostSeconds)
					departsLater     = transferFromTime >= mapStopToArrivalDetails[c.FromSSIDU].Time
					arrivesEarlier   = transferToST < mapStopToArrivalDetails[transferToSSIDU].Time
					tPoss            = transferToStopCostSeconds < maxTransferSeconds
				)
				if departsLater && arrivesEarlier && tPoss && mapStopToArrivalDetails[c.FromSSIDU].NTrips < maxNTrips {
					mapStopToArrivalDetails[transferToSSIDU] = ArrivalDetails{
						Time:       transferToST,
						Connection: &cc,
						NTrips:     mapStopToArrivalDetails[c.FromSSIDU].NTrips + 1,
					}
				}
			}
		}
	}
	utillog.Warnf("Completed main CSA loop %v", mapStopToArrivalDetails)
	return mapStopToArrivalDetails
}
